/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "mograsoJobFactory.h"

mograsoJobFactory::mograsoJobFactory()
{
	f_fileHandler = 0;
	f_numberOfJobs = 0;
	f_variableStorage = 0;
	f_layers = 0;
	f_activeJob = 0;
	f_type = L"";
	f_sections = 0;
	f_outputStorage = 0;
	f_options = 0;
}

mograsoJobFactory::~mograsoJobFactory()
{
	delete f_fileHandler;
	delete f_variableStorage;
	delete f_layers;
	delete f_sections;
	delete f_outputStorage;
	delete f_options;
}

void mograsoJobFactory::set_activeJob(const int& jobNumber)
{
	// Unset everthing
	delete f_variableStorage;
	f_variableStorage = 0;
	delete f_layers;
	f_layers = 0;
	delete f_sections;
	f_sections = 0;
	f_activeJob = 0;
	delete f_outputStorage;
	f_outputStorage = 0;
	delete f_options;
	f_options = 0;

	// Set new objects
	f_activeJob = jobNumber;
	f_type = f_fileHandler->get_jobType(f_activeJob);
	f_variableStorage = new variableStorage(f_fileHandler->get_variableStorage(f_activeJob));
	f_layers = new std::vector<layer>(f_fileHandler->get_layerVector(f_activeJob));
	f_options = new std::map<std::wstring, std::wstring>(f_fileHandler->get_options(f_activeJob));
	
	if (f_type == L"spectrum")
		f_sections = new std::vector<section>(f_fileHandler->get_sectionVector(f_activeJob));

	f_outputStorage = new outputStorage();

	if ((f_options->count(L"variableType") > 0) && (f_options->find(L"variableType")->second == L"longDouble"))
	{
		if (f_type == L"mode_structure")
		{
			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobModeStructure<modeModelLP<long double>, long double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobModeStructure<modeModelLP<long double>, long double>::perVarSetFilenames(), f_outputStorage);
			}
			else
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobModeStructure<modeModelHE<long double>, long double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobModeStructure<modeModelHE<long double>, long double>::perVarSetFilenames(), f_outputStorage);
			}
		}
		else if (f_type == L"spectrum")
		{
			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobSpectrum<modeModelLP<long double>, long double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobSpectrum<modeModelLP<long double>, long double>::perVarSetFilenames(), f_outputStorage);
			}
			else
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobSpectrum<modeModelHE<long double>, long double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobSpectrum<modeModelHE<long double>, long double>::perVarSetFilenames(), f_outputStorage);
			}		
		}
	}
	else
	{
		if (f_type == L"mode_structure")
		{
			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobModeStructure<modeModelLP<double>, double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobModeStructure<modeModelLP<double>, double>::perVarSetFilenames(), f_outputStorage);
			}
			else
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobModeStructure<modeModelHE<double>, double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobModeStructure<modeModelHE<double>, double>::perVarSetFilenames(), f_outputStorage);
			}
		}
		else if (f_type == L"spectrum")
		{
			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobSpectrum<modeModelLP<double>, double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobSpectrum<modeModelLP<double>, double>::perVarSetFilenames(), f_outputStorage);
			}
			else
			{
				f_fileHandler->get_perJobFiles(f_activeJob, mograsoJobSpectrum<modeModelHE<double>, double>::perJobFilenames(), f_outputStorage);
				f_fileHandler->get_perVarSetFiles(f_activeJob, mograsoJobSpectrum<modeModelHE<double>, double>::perVarSetFilenames(), f_outputStorage);
			}		
		}
	}
}

void mograsoJobFactory::process_file(const std::wstring& filename)
{
	f_fileHandler = new mograsoFileHandler(filename);
	f_numberOfJobs = f_fileHandler->get_numberOfJobs();
	
	for (int jobCounter = 0; jobCounter < f_numberOfJobs; jobCounter++)
	{
		set_activeJob(jobCounter);
		jobServer::set_jobFactory(this);
		jobServer::do_jobs(2);
	}

	delete f_fileHandler;
	f_fileHandler = 0;
	f_numberOfJobs = 0;
}

unsigned long mograsoJobFactory::get_numberOfJobs() const
{
	if (f_variableStorage != 0)
		return f_variableStorage->get_numberOfVarSets();
	else
		return 0;
}

jobInterface* mograsoJobFactory::checkout_job(const unsigned long& number) const
{
	if ((f_options->count(L"variableType") > 0) && (f_options->find(L"variableType")->second == L"longDouble"))
	{
		if (f_type == L"mode_structure")
		{
			std::map<std::wstring, std::complex<double> > varSet = f_variableStorage->get_varSet(number);
			std::vector<layerDescription<long double> > layers;

			for (std::vector<layer>::iterator it = f_layers->begin(); it != f_layers->end(); it++)
			{
				layer temp(*it);
				temp.set_vars(varSet);
				layers.push_back(temp.description<long double>());
			}

			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
				return new mograsoJobModeStructure<modeModelLP<long double>, long double>(varSet, layers, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			else
			{
				f_options->insert(std::pair<std::wstring, std::wstring>(L"modeModel", L"hybrid"));
				return new mograsoJobModeStructure<modeModelHE<long double>, long double>(varSet, layers, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			}
		}
		else if (f_type == L"spectrum")
		{
			std::map<std::wstring, std::complex<double> > varSet = f_variableStorage->get_varSet(number);
			std::vector<layerDescription<long double> > layers;

			for (std::vector<layer>::iterator it = f_layers->begin(); it != f_layers->end(); it++)
			{
				layer temp(*it);
				temp.set_vars(varSet);
				layers.push_back(temp.description<long double>());
			}

			std::vector<section> sections = *f_sections;
			for (std::vector<section>::iterator it = sections.begin(); it != sections.end(); it++)
				it->set_vars(varSet);

			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
				return new mograsoJobSpectrum<modeModelLP<long double>, long double>(varSet, layers, sections, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			else
			{
				f_options->insert(std::pair<std::wstring, std::wstring>(L"modeModel", L"hybrid"));
				return new mograsoJobSpectrum<modeModelHE<long double>, long double>(varSet, layers, sections, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			}
		}
	}
	else
	{
		if (f_type == L"mode_structure")
		{
			std::map<std::wstring, std::complex<double> > varSet = f_variableStorage->get_varSet(number);
			std::vector<layerDescription<double> > layers;

			for (std::vector<layer>::iterator it = f_layers->begin(); it != f_layers->end(); it++)
			{
				layer temp(*it);
				temp.set_vars(varSet);
				layers.push_back(temp.description<double>());
			}

			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
				return new mograsoJobModeStructure<modeModelLP<double>, double>(varSet, layers, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			else
			{
				f_options->insert(std::pair<std::wstring, std::wstring>(L"modeModel", L"hybrid"));
				return new mograsoJobModeStructure<modeModelHE<double>, double>(varSet, layers, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			}
		}
		else if (f_type == L"spectrum")
		{
			std::map<std::wstring, std::complex<double> > varSet = f_variableStorage->get_varSet(number);
			std::vector<layerDescription<double> > layers;

			for (std::vector<layer>::iterator it = f_layers->begin(); it != f_layers->end(); it++)
			{
				layer temp(*it);
				temp.set_vars(varSet);
				layers.push_back(temp.description<double>());
			}

			std::vector<section> sections = *f_sections;
			for (std::vector<section>::iterator it = sections.begin(); it != sections.end(); it++)
				it->set_vars(varSet);

			if ((f_options->count(L"modeModel") > 0) && (f_options->find(L"modeModel")->second == L"LP"))
				return new mograsoJobSpectrum<modeModelLP<double>, double>(varSet, layers, sections, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			else
			{
				f_options->insert(std::pair<std::wstring, std::wstring>(L"modeModel", L"hybrid"));
				return new mograsoJobSpectrum<modeModelHE<double>, double>(varSet, layers, sections, f_outputStorage->checkout_fileMap(f_variableStorage->get_filenameStringForVarSet(number)), f_variableStorage->get_lineStartForVarSet(number), *f_options);
			}
		}
	}
	
	return 0;
}

void mograsoJobFactory::checkin_job(jobInterface* job) const
{
	f_outputStorage->checkin_fileMap(job->get_fileMap());
	delete job;
}
