/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _MOGRASOJOBFACTORY_H
#define _MOGRASOJOBFACTORY_H 1

#include <map>
#include <string>

#include "jobServer.h"
#include "mograsoFileHandler.h"
#include "modeSolver.h"
#include "modeModelHE.h"
#include "modeModelLP.h"
#include "gratingModelCMT.h"
#include "mograsoJobModeStructure.h"
#include "mograsoJobSpectrum.h"
#include "section.h"

class mograsoJobFactory : public jobFactoryInterface
{
	private:
		mograsoFileHandler* f_fileHandler;
		unsigned long f_numberOfJobs;
		int f_activeJob;
		std::wstring f_type;
		variableStorage* f_variableStorage;
		std::vector<layer>* f_layers;
		std::vector<section>* f_sections;
		outputStorage* f_outputStorage;
		std::map<std::wstring, std::wstring>* f_options;
	
	public:
		mograsoJobFactory();
		~mograsoJobFactory();
		void set_activeJob(const int& jobNumber);
		void process_file(const std::wstring& filename);
		virtual unsigned long get_numberOfJobs() const;
		virtual jobInterface* checkout_job(const unsigned long& number) const;
		virtual void checkin_job(jobInterface* job) const;
		
	private:
		mograsoJobFactory(const mograsoJobFactory&);
};

#endif

