/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <complex>

#include "algorithms.h"

std::complex<double> intFunc(const double& x, void* param);
std::complex<double> intFunc2(const double& x, void* param);
std::complex<long double> intFunc(const long double& x, void* param);
std::complex<long double> intFunc2(const long double& x, void* param);

int main()
{
	int ret = 0;
	unsigned int check = 0;
	
	// Test integral using doubles
	double start = 0.0;
	double end = M_PI / 2.0;
	
	ret = ret | !thresholdEqual(integral(intFunc, 0, start, end, check, 1e-15), 1.0 / 24.0, 5e-14);

	start = 0.0;
	end = 2.0;
	ret = ret | !thresholdEqual(integral(intFunc2, 0, start, end, check, 1e-15), -std::cos(3000.0)/3000.0 + std::cos(1000.0) / 1000.0 - 1.0 / 1500.0, 1e-13);
	
	// Test integral using long doubles
	long double startL = 0.0L;
	long double endL = M_PI / 2.0L;
	
	ret = ret | !thresholdEqual(integral(intFunc, 0, startL, endL, check, 1e-15L), 1.0L / 24.0L, 5e-14L);

	startL = 0.0L;
	endL = 2.0L;
	ret = ret | !thresholdEqual(integral(intFunc2, 0, startL, endL, check, 1e-15L), -std::cos(3000.0L)/3000.0L + std::cos(1000.0L) / 1000.0L - 1.0L / 1500.0L, 1e-13L);
	
	return ret;
}

std::complex<double> intFunc(const double& x, void* param)
{
	return std::cos(x) * std::cos(x) * std::cos(x) * std::sin(x) * std::sin(x) * std::sin(x) * std::sin(x) * std::sin(x);
}

std::complex<double> intFunc2(const double& x, void* param)
{
	return std::cos(x * 1000.0) * std::sin(x * 500);
}

std::complex<long double> intFunc(const long double& x, void* param)
{
	return std::cos(x) * std::cos(x) * std::cos(x) * std::sin(x) * std::sin(x) * std::sin(x) * std::sin(x) * std::sin(x);
}

std::complex<long double> intFunc2(const long double& x, void* param)
{
	return std::cos(x * 1000.0L) * std::sin(x * 500L);
}

