/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "modeModelHE.h"

int main()
{
	int ret = 0;

	ret = ret | !thresholdEqual(modeModelHE<double>::get_core_mode_order(), 1.0);

	std::complex<double> nEff(1.448);
	std::complex<double> e1(1.45);
	std::complex<double> u1(1.0);
	std::complex<double> e2(1.447);
	std::complex<double> u2(1.0);
	double r = 4.1e-6;
	double lambda = 1.55e-6;
	double order = 1.0;
	double scaling = 1e0;

	matrix<std::complex<double> > Mmatrix = modeModelHE<double>::M(nEff, e1, u1, e2, u2, r, lambda, order, scaling);

	ret = ret | !thresholdEqual(Mmatrix(0,0), std::complex<double>(-0.3510921809804917,0));
	ret = ret | !thresholdEqual(Mmatrix(0,1), std::complex<double>(3.12427915486603,0));
	ret = ret | !thresholdEqual(Mmatrix(0,2), std::complex<double>(0,-346.036858322947));
	ret = ret | !thresholdEqual(Mmatrix(0,3), std::complex<double>(0,-507.8126121980532));

	ret = ret | !thresholdEqual(Mmatrix(1,0), std::complex<double>(-1.175422237345316,-7.197147701385941e-17));
	ret = ret | !thresholdEqual(Mmatrix(1,1), std::complex<double>(1.539478053328175,9.426273028378936e-17));
	ret = ret | !thresholdEqual(Mmatrix(1,2), std::complex<double>(1.900431464739065e-14,-310.3742617057681));
	ret = ret | !thresholdEqual(Mmatrix(1,3), std::complex<double>(2.788902520643767e-14,-455.4773886218595));

	ret = ret | !thresholdEqual(Mmatrix(2,0), std::complex<double>(0,0.005126219885111542));
	ret = ret | !thresholdEqual(Mmatrix(2,1), std::complex<double>(0,0.007522779865636847));
	ret = ret | !thresholdEqual(Mmatrix(2,2), std::complex<double>(-0.3554805250135581,0));
	ret = ret | !thresholdEqual(Mmatrix(2,3), std::complex<double>(3.132946126124557,0));

	ret = ret | !thresholdEqual(Mmatrix(3,0), std::complex<double>(-2.815315574199634e-19,0.004597911100840132));
	ret = ret | !thresholdEqual(Mmatrix(3,1), std::complex<double>(-4.13150426467553e-19,0.006747481346605497));
	ret = ret | !thresholdEqual(Mmatrix(3,2), std::complex<double>(-1.179358318160449,-7.221248449262793e-17));
	ret = ret | !thresholdEqual(Mmatrix(3,3), std::complex<double>(1.547251805577075,9.47387196036416e-17));

	matrix<AD<std::complex<double> > > MmatrixAD = modeModelHE<double>::MAD(nEff, e1, u1, e2, u2, r, lambda, order, scaling);

	ret = ret | !thresholdEqual(Mmatrix(0,0), MmatrixAD(0,0).fun());
	ret = ret | !thresholdEqual(Mmatrix(0,1), MmatrixAD(0,1).fun());
	ret = ret | !thresholdEqual(Mmatrix(0,2), MmatrixAD(0,2).fun());
	ret = ret | !thresholdEqual(Mmatrix(0,3), MmatrixAD(0,3).fun());

	ret = ret | !thresholdEqual(Mmatrix(1,0), MmatrixAD(1,0).fun());
	ret = ret | !thresholdEqual(Mmatrix(1,1), MmatrixAD(1,1).fun());
	ret = ret | !thresholdEqual(Mmatrix(1,2), MmatrixAD(1,2).fun());
	ret = ret | !thresholdEqual(Mmatrix(1,3), MmatrixAD(1,3).fun());

	ret = ret | !thresholdEqual(Mmatrix(2,0), MmatrixAD(2,0).fun());
	ret = ret | !thresholdEqual(Mmatrix(2,1), MmatrixAD(2,1).fun());
	ret = ret | !thresholdEqual(Mmatrix(2,2), MmatrixAD(2,2).fun());
	ret = ret | !thresholdEqual(Mmatrix(2,3), MmatrixAD(2,3).fun());

	ret = ret | !thresholdEqual(Mmatrix(3,0), MmatrixAD(3,0).fun());
	ret = ret | !thresholdEqual(Mmatrix(3,1), MmatrixAD(3,1).fun());
	ret = ret | !thresholdEqual(Mmatrix(3,2), MmatrixAD(3,2).fun());
	ret = ret | !thresholdEqual(Mmatrix(3,3), MmatrixAD(3,3).fun());

	ret = ret | !thresholdEqual(MmatrixAD(0,0).diff(), std::complex<double>(723.5580061313912,0));
	ret = ret | !thresholdEqual(MmatrixAD(0,1).diff(), std::complex<double>(-4403.437127350549,0));
	ret = ret | !thresholdEqual(MmatrixAD(0,2).diff(), std::complex<double>(0,1928.012395622303));
	ret = ret | !thresholdEqual(MmatrixAD(0,3).diff(), std::complex<double>(0,713987.8867551406));

	ret = ret | !thresholdEqual(MmatrixAD(1,0).diff(), std::complex<double>(646.864497984208,3.960771871467918e-14));
	ret = ret | !thresholdEqual(MmatrixAD(1,1).diff(), std::complex<double>(-3945.424424786516,-2.415795909559783e-13));
	ret = ret | !thresholdEqual(MmatrixAD(1,2).diff(), std::complex<double>(-1.035883464399515e-11,169178.1952896455));
	ret = ret | !thresholdEqual(MmatrixAD(1,3).diff(), std::complex<double>(-5.425845974590555e-11,886137.1587130515));

	ret = ret | !thresholdEqual(MmatrixAD(2,0).diff(), std::complex<double>(0,-0.02856174203256975));
	ret = ret | !thresholdEqual(MmatrixAD(2,1).diff(), std::complex<double>(0,-10.57707817760018));
	ret = ret | !thresholdEqual(MmatrixAD(2,2).diff(), std::complex<double>(725.0884324624338,0));
	ret = ret | !thresholdEqual(MmatrixAD(2,3).diff(), std::complex<double>(-4417.852773869004,0));

	ret = ret | !thresholdEqual(MmatrixAD(3,0).diff(), std::complex<double>(1.53456670471421e-16,-2.506220386533763));
	ret = ret | !thresholdEqual(MmatrixAD(3,1).diff(), std::complex<double>(8.037895056410546e-16,-13.12731235032674));
	ret = ret | !thresholdEqual(MmatrixAD(3,2).diff(), std::complex<double>(650.3607389906672,3.982179466222938e-14));
	ret = ret | !thresholdEqual(MmatrixAD(3,3).diff(), std::complex<double>(-3962.548381868264,-2.426280962882229e-13));

	return ret;
}

