/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "layerStandIn.h"

layerStandIn::layerStandIn(const std::wstring& type) : f_type(type)
{
	layer temp(type);
	std::map<std::wstring, std::complex<double> > tempMap = temp.get_vars();

	for (std::map<std::wstring, std::complex<double> >::const_iterator it = tempMap.begin(); it != tempMap.end(); it++)
		f_varsOrig.push_back(it->first);
}

layerStandIn::layerStandIn(const layerStandIn& layerSI)
{
	f_type = layerSI.f_type;
	f_varsOrig = layerSI.f_varsOrig;
	f_varsSet = layerSI.f_varsSet;
	f_varsRen = layerSI.f_varsRen;
}

void layerStandIn::set_varString(const std::wstring& varName, const std::wstring& varValue)
{
	if (f_varsSet.count(varName) > 0)
		f_varsSet.erase(f_varsSet.find(varName));
	if (f_varsRen.count(varName) > 0)
		f_varsRen.erase(f_varsRen.find(varName));

	std::complex<double> temp = wstringToComplexDouble(varValue);

	if (!std::isnan(std::real(temp)) && !std::isnan(imag(temp)) && (varValue.length() != 0))
		f_varsSet[varName] = temp;
	else
		f_varsRen[varName] = varValue;
}

void layerStandIn::rename_var(const std::wstring& varName, const std::wstring& varValue)
{
	if (f_varsSet.count(varName) > 0)
		f_varsSet.erase(f_varsSet.find(varName));
	if (f_varsRen.count(varName) > 0)
		f_varsRen.erase(f_varsRen.find(varName));

	f_varsRen[varName] = varValue;
}

void layerStandIn::set_var(const std::wstring& varName, const std::complex<double>& varValue)
{
	if (f_varsSet.count(varName) > 0)
		f_varsSet.erase(f_varsSet.find(varName));
	if (f_varsRen.count(varName) > 0)
		f_varsRen.erase(f_varsRen.find(varName));

	f_varsSet[varName] = varValue;
}

std::wstring layerStandIn::get_varString(const std::wstring& var) const
{
	if (f_varsSet.count(var) > 0)
		return complexDoubleToWString(f_varsSet.find(var)->second);
	else if (f_varsRen.count(var) > 0)
		return f_varsRen.find(var)->second;
	else
		return std::wstring();
}

std::wstring layerStandIn::get_rString() const
{
	try
	{
		eo<std::complex<double> > r = layer(f_type).get_r();
		r.set_vars(f_varsSet);
		return complexDoubleToWString((std::complex<double>)r);
	}
	catch (...)
	{
		return std::wstring(L"variable");
	}
}

std::wstring layerStandIn::get_type() const
{
	return f_type;
}

std::vector<std::wstring> layerStandIn::get_varsOrig() const
{
	return f_varsOrig;
}

layer layerStandIn::get_layer() const
{
	layer temp(f_type);
	temp.rename_vars(f_varsRen);
	temp.set_vars(f_varsSet);
	return temp;
}

