/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _OUTPUTSTORAGE_H
#define _OUTPUTSTORAGE_H 1

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef HAVE_LIBPTHREAD
	#include <pthread.h>
#endif

#include <map>
#include <iostream>
#include <fstream>
#include <string>

#include "pointerWrapper_template.h"
#include "stringFunctions.h"

class outputStorage
{
	private:
		std::map<std::wstring, pointerWrapper<std::ofstream> > f_perJob;
		std::map<std::wstring, std::wstring> f_perVarSet;
	
	public:
		outputStorage();
		~outputStorage();
		void add_perJobFile(const std::wstring& dataId, const std::wstring& filename);
		void add_perVarSetFile(const std::wstring& dataId, const std::wstring& filename);
		std::map<std::wstring, pointerWrapper<std::ofstream> > checkout_fileMap(const std::wstring& filenameSuffix) const;
		void checkin_fileMap(std::map<std::wstring, pointerWrapper<std::ofstream> >& map) const;
	
	private:
		outputStorage(const outputStorage&);
		std::wstring incorporateFilenameSuffix(const std::wstring& filenameSuffix, const std::wstring& filename) const;
};
#endif
