/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "variableStandIn.h"

variableStandIn::variableStandIn(const std::wstring& type) : f_type(type), f_name(L""), f_value(NAN), f_start(NAN), f_end(NAN), f_step(NAN), f_stepType(L""), f_prefix(L""), f_suffix(L"")
{
}

variableStandIn::variableStandIn(const variableStandIn& variableSI)
{
	f_type = variableSI.f_type;
	f_name = variableSI.f_name;
	f_prefix = variableSI.f_prefix;
	f_suffix = variableSI.f_suffix;
	f_value = variableSI.f_value;
	f_start = variableSI.f_start;
	f_end = variableSI.f_end;
	f_step = variableSI.f_step;
	f_stepType = variableSI.f_stepType;
}

variableStandIn::variableStandIn(const std::wstring& name, const std::wstring& prefix, const std::wstring& suffix, const std::complex<double>& value, const std::complex<double>& start, const std::complex<double>& end, const std::complex<double>& step, const std::complex<double>& numberOfSteps) : f_name(name), f_prefix(prefix), f_suffix(suffix), f_value(value), f_start(start), f_end(end), f_step(step)
{
	if (!std::isnan(std::real(value)) && !std::isnan(std::imag(value)))
		f_type = L"value";
	else
		f_type = L"interval";

	if (!std::isnan(std::real(step)) && !std::isnan(std::imag(step)))
		f_stepType = L"step";
	else
	{
		f_stepType = L"numberOfSteps";
		f_step = numberOfSteps;
	}
}

void variableStandIn::set_varString(const std::wstring& varName, const std::wstring& varValue)
{
	if (varName == L"type")
		f_type = varValue;
	else if (varName == L"name")
		f_name = varValue;
	else if (varName == L"stepType")
		f_stepType = varValue;
	else if (varName == L"prefix")
		f_prefix = varValue;
	else if (varName == L"suffix")
		f_suffix = varValue;
	else if (varName == L"value")
		f_value = wstringToComplexDouble(varValue);
	else if (varName == L"start")
		f_start = wstringToComplexDouble(varValue);
	else if (varName == L"end")
		f_end = wstringToComplexDouble(varValue);
	else if (varName == L"step")
		f_step = wstringToComplexDouble(varValue);
}

void variableStandIn::set_var(const std::wstring& varName, const std::complex<double>& varValue)
{
	if (varName == L"start")
		f_start = varValue;
	else if (varName == L"value")
		f_value = varValue;
	else if (varName == L"end")
		f_end = varValue;
	else if (varName == L"step")
		f_step = varValue;
}

std::wstring variableStandIn::get_varString(const std::wstring& var) const
{
	if (var == L"type")
		return f_type;
	else if (var == L"name")
		return f_name;
	else if (var == L"stepType")
		return f_stepType;
	else if (var == L"prefix")
		return f_prefix;
	else if (var == L"suffix")
		return f_suffix;
	else if ((var == L"value") && !std::isnan(std::real(f_value)) && !std::isnan(std::imag(f_value)))
		return complexDoubleToWString(f_value);
	else if ((var == L"start") && (!std::isnan(std::real(f_start)) && !std::isnan(std::imag(f_start))))
		return complexDoubleToWString(f_start);
	else if ((var == L"end") && !std::isnan(std::real(f_end)) && !std::isnan(std::imag(f_end)))
		return complexDoubleToWString(f_end);
	else if ((var == L"step") && !std::isnan(std::real(f_step)) && !std::isnan(std::imag(f_step)))
		return complexDoubleToWString(f_step);
	else
		return std::wstring();
}

std::wstring variableStandIn::get_valueString() const
{
	if (f_type == L"value")
		return complexDoubleToWString(f_value);
	else if (f_type == L"interval")
	{
		std::wstring output(complexDoubleToWString(f_start));
		output += L" - " + complexDoubleToWString(f_end);
		if (f_stepType == L"step")
			output += L", step: " + complexDoubleToWString(f_step);
		else
			output += L", " + complexDoubleToWString(f_step) + L" steps";
		return output;
	}
	return std::wstring();
}

std::wstring variableStandIn::get_type() const
{
	return f_type;
}

std::wstring variableStandIn::get_name() const
{
	return f_name;
}

std::wstring variableStandIn::get_prefix() const
{
	return f_prefix;
}

std::wstring variableStandIn::get_suffix() const
{
	return f_suffix;
}

std::wstring variableStandIn::get_stepType() const
{
	return f_stepType;
}

std::complex<double> variableStandIn::get_value() const
{
	return f_value;
}

std::complex<double> variableStandIn::get_start() const
{
	return f_start;
}

std::complex<double> variableStandIn::get_end() const
{
	return f_end;
}

std::complex<double> variableStandIn::get_step() const
{
	return f_step;
}

