/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _JOBSERVER_H
#define _JOBSERVER_H 1

#include <iostream>
#include <ctime>
#include <string>
#include <map>

#include "pointerWrapper_template.h"
#include "stringFunctions.h"

class jobInterface
{
	public:
		virtual void job() = 0;
		virtual std::map<std::wstring, pointerWrapper<std::ofstream> >& get_fileMap() = 0;
		virtual ~jobInterface() {};
};

class jobFactoryInterface
{
	public:
		virtual unsigned long get_numberOfJobs() const = 0;
		virtual jobInterface* checkout_job(const unsigned long& number) const = 0;
		virtual void checkin_job(jobInterface* job) const = 0;
};

class jobServer
{
	public:
		static unsigned long f_current;
		static unsigned long f_stepsCompleted;
		static int f_threadCount;
		static jobFactoryInterface* f_jobFactory;
		static time_t f_startTime;
		static pthread_mutex_t f_timing_mutex;
		static pthread_mutex_t f_jobFactory_mutex;

	public:
		static void set_jobFactory(jobFactoryInterface* jobFactory);
		static void do_jobs(const int& numberOfThreads = 3);
};

class jobClient : public jobServer
{
	private:
		pthread_t _thread;
		int _id;
	
	public:
		pthread_t* get_threadPtr();
		static void* process(void* param);
		void set_id(int id);
		const int& get_id() const;
};

class jobSin : public jobInterface
{
	public:
		void job();
};
#endif
