/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _POINTERWRAPPER_H
#define _POINTERWRAPPER_H 1

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef HAVE_LIBPTHREAD
	#include <pthread.h>
#endif

template <class T>
class pointerWrapper
{
	private:
		T* _object_ptr;	
#ifdef HAVE_LIBPTHREAD
		pthread_mutex_t* _mutex_ptr;
#endif

	public:
#ifndef HAVE_LIBPTHREAD
		pointerWrapper(T* objectPtr);
#endif
#ifdef HAVE_LIBPTHREAD
		pointerWrapper(T* objectPtr, pthread_mutex_t* mutexPtr);
#endif
		~pointerWrapper();
		T* getObjectPtr();
#ifdef HAVE_LIBPTHREAD
		pthread_mutex_t* getMutexPtr();
#endif
		void lock();
		void unlock();
};

#ifndef HAVE_LIBPTHREAD
template<class T>
pointerWrapper<T>::pointerWrapper(T* objectPtr)
{
	_object_ptr = objectPtr;
}
#endif

#ifdef HAVE_LIBPTHREAD
template<class T>
pointerWrapper<T>::pointerWrapper(T* objectPtr, pthread_mutex_t* mutexPtr)
{
	_object_ptr = objectPtr;
	_mutex_ptr = mutexPtr;
}
#endif

template<class T>
pointerWrapper<T>::~pointerWrapper()
{
}

template<class T>
T* pointerWrapper<T>::getObjectPtr()
{
	return _object_ptr;
}

#ifdef HAVE_LIBPTHREAD
template<class T>
pthread_mutex_t* pointerWrapper<T>::getMutexPtr()
{
	return _mutex_ptr;
}
#endif

template<class T>
void pointerWrapper<T>::lock()
{
#ifdef HAVE_LIBPTHREAD
	if (_mutex_ptr != NULL)
		pthread_mutex_lock(_mutex_ptr);
#endif
}

template<class T>
void pointerWrapper<T>::unlock()
{
#ifdef HAVE_LIBPTHREAD
	if (_mutex_ptr != NULL)
		pthread_mutex_unlock(_mutex_ptr);
#endif
}
#endif
