/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <complex>

#include "bessel_amos.h"

template <class T>
bool thresholdEqual(const T& num1, const T& num2, const T& threshold = (T)1e-15)
{
	if (num1 == 0.0)
		if (std::abs(num2) < threshold)
			return true;
		else
			return false;

	if (num2 == 0.0)
		if (std::abs(num1) < threshold)
			return true;
		else
			return false;
	
	T ratio = std::abs(num1 / num2);
	if (ratio == 1.0)
		return true;
	else if (ratio < 1.0)
		ratio = 1.0 - ratio;
	else if (ratio > 1.0)
		ratio--;
	
	if (ratio < threshold)
		return true;
	else
		return false;
}

template<class T>
bool thresholdEqual(const std::complex<T>& num1, const std::complex<T>& num2, const T& threshold = (T)1e-15)
{
	if (thresholdEqual(std::real(num1), std::real(num2), threshold) && thresholdEqual(std::real(num1), std::real(num2), threshold))
		return true;
	else
		return false;
}

int main()
{
	int ret = 0;
	
	double order = 0.0;
	std::complex<double> arg(2.0, 0.05);
	std::complex<double> J = besselj(order, arg);
	std::complex<double> Jp = order / arg * J - besselj(order + 1, arg);
	std::complex<double> Y = bessely(order, arg);
	std::complex<double> Yp = order / arg * Y - bessely(order + 1, arg);

	ret = ret | !thresholdEqual(2.0 / (M_PI * arg), J * Yp - Jp * Y, 5e-13);

	std::complex<long double> argL((long double)2.0, (long double)0.05);
	std::complex<long double> JL = besselj(order, argL);
	std::complex<long double> JpL = (long double)order / argL * JL - besselj(order + 1, argL);
	std::complex<long double> YL = bessely(order, arg);
	std::complex<long double> YpL = (long double)order / argL * YL - bessely(order + 1, argL);

	ret = ret | !thresholdEqual((long double)2.0 / ((long double)M_PI * argL), JL * YpL - JpL * YL, (long double)1e-13);

	arg = std::complex<double>(3.0, 0.02);
	order = 1.0;
	J = besselj(order, arg);
	Jp = order / arg * J - besselj(order + 1, arg);
	Y = bessely(order, arg);
	Yp = order / arg * Y - bessely(order + 1, arg);

	ret = ret | !thresholdEqual(2.0 / (M_PI * arg), J * Yp - Jp * Y, 1e-13);

	ret = ret | !thresholdEqual(besselk(0, arg) * besseli(1, arg) + besseli(0, arg) * besselk(1, arg), 1.0 / arg);

	ret = ret | !thresholdEqual(besselh1(0, arg) * besselj(1, arg) - besselj(0, arg) * besselh1(1, arg), std::complex<double>(0.0, 2.0) / (M_PI * arg), 5e-14);

	return ret;
}
